@@ -91,8 +91,9 @@ class V3BackendFunctionProvider < LookupKeyFunctionProvider
     @backend ||= instantiate_backend(lookup_invocation)
     config = parent_data_provider.config(lookup_invocation)
 
-    # Never pass hiera.yaml defined merge_behavior down to the backend. It will pick it up from the config
-    resolution_type = lookup_invocation.hiera_v3_merge_behavior? ? nil : convert_merge(merge)
+    # A merge_behavior retrieved from hiera.yaml must not be converted here. Instead, passing the symbol :hash
+    # tells the V3 backend to pick it up from the config.
+    resolution_type = lookup_invocation.hiera_v3_merge_behavior? ? :hash : convert_merge(merge)
     @backend.lookup(key, lookup_invocation.scope, lookup_invocation.hiera_v3_location_overrides, resolution_type, context = {:recurse_guard => nil})
   end
 
